@@ -19,10 +19,10 @@ NUM_WORKERS = 2
 PIN_MEMORY = True
 MAX_EPOCH = 50
 DEVICE = 'cuda' if torch.cuda.is_available() else 'cpu'
-SMOOTH_FACTOR = 0.2
+SMOOTH_FACTOR = 0.5
 
 
-def train_fn(loader, model, optimizer, loss_fn, scaler, device):
+def train_fn(loader, model, optimizer, loss_fn, scaler, clip_max_norm, device):
     loop = tqdm(loader)
     smooth_loss = None
 
@@ -38,6 +38,8 @@ def train_fn(loader, model, optimizer, loss_fn, scaler, device):
         # backward
         optimizer.zero_grad()
         scaler.scale(loss).backward()
+        if clip_max_norm is not None:
+            torch.nn.utils.clip_grad_norm_(model.parameters(), 1, norm_type=2)
         scaler.step(optimizer)
         scaler.update()
 
@@ -49,7 +51,10 @@ def train_fn(loader, model, optimizer, loss_fn, scaler, device):
         loop.set_postfix(loss=loss, smooth_loss=smooth_loss)
 
 
-def loop_fn(model, optimizer, loss_fn, trainset, valset, batch_size, max_patience=10, checkpoint=None, model_name='my_model'):
+def loop_fn(
+    model, optimizer, loss_fn, trainset, valset, batch_size,
+    scheduler=None, clip_max_norm=None, max_patience=10, checkpoint=None, model_name='my_model'
+):
     if checkpoint is not None:
         load_checkpoint(model, optimizer, checkpoint)
 
@@ -58,8 +63,10 @@ def loop_fn(model, optimizer, loss_fn, trainset, valset, batch_size, max_patienc
     scaler = torch.cuda.amp.GradScaler()
 
     for epoch in range(MAX_EPOCH):
-        train_fn(train_loader, model, optimizer, loss_fn, scaler, DEVICE)
+        train_fn(train_loader, model, optimizer, loss_fn, scaler, clip_max_norm, DEVICE)
         loss = check_loss(val_loader, model, DEVICE, epoch)
+        if scheduler is not None:
+            scheduler.step()
         # early stop
         if loss < min_loss:
             save_checkpoint(model, optimizer, file_name=f'{model_name}.pth.tar')
