@@ -20,7 +20,7 @@ def load_checkpoint(model, optimizer, file_name='my_checkpoint.pth.tar'):
     optimizer.load_state_dict(checkpoint['optimizer'])
 
 
-def check_loss(dataloader, model, loss_fn, device):
+def check_loss(dataloader, model, loss_fn, device, epoch='-'):
     model.eval()
     loss = 0
 
@@ -33,7 +33,7 @@ def check_loss(dataloader, model, loss_fn, device):
             loss += loss_fn(predictions, targets).item()
 
     print(
-        f'Average {loss_fn} over {len(dataloader)} batches: {loss/len(dataloader):.4f}'
+        f'Average {loss_fn} @epoch{epoch}: {loss/len(dataloader):.4f}'
     )
     model.train()
     return loss
